# Generated by Django 4.2.6 on 2024-06-07 07:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0007_alter_user_name_alter_user_username'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='last_path',
            field=models.CharField(blank=True, default='/start', max_length=250, null=True),
        ),
        migrations.AddField(
            model_name='user',
            name='path',
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
    ]



# Generated by Django 4.2.6 on 2024-07-18 21:56

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0008_user_last_path_user_path'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='created',
            field=models.DateTimeField(default=datetime.datetime.now, verbose_name='Creation date'),
        ),
    ]



class User:
    id: int
    is_bot: bool
    first_name: str
    last_name: str
    username: str

    def __init__(self, user_data):
        self.id = user_data.get('id')
        self.is_bot = user_data.get('is_bot')
        self.first_name = user_data.get('first_name')
        self.last_name = user_data.get('last_name')
        self.username = user_data.get('username')


class Chat:
    id: int
    username: str
    type: str

    def __init__(self, chat_data):
        self.id = chat_data.get('id')
        self.username = chat_data.get('username')
        self.type = chat_data.get('type')


class Message:
    id: int
    update_id: int
    user: User
    chat: Chat
    date: int
    text: str
    data: str
    document: dict
    photo: dict

    def __init__(self, content):
        self.update_id = content.get('update_id')
        key = 'callback_query' if 'callback_query' in content else 'message'
        if key in content:
            message_data = content[key]
            if message_data:
                print(message_data)
                self.id = message_data.get('message_id') or message_data['message']['message_id']
                self.date = message_data.get('date')
                self.text = message_data.get('text')
                self.data = message_data.get('data')
                self.photo = message_data.get('photo')
                self.voice = message_data.get('voice')
                self.document = message_data.get('document')
                self.user = User(message_data.get('from'))
                self.chat = Chat(message_data.get('chat') or message_data['message']['chat'])



import json
import re

from oscarbot.bot_logger import log


class Button:

    def __init__(self, text, callback=None, url=None, web_app=None):
        self.text = text
        self.callback = callback
        self.url = url
        self.web_app = web_app

    def build(self):
        menu_button = {
            'text': self.text
        }
        if self.callback is not None:
            menu_button['callback_data'] = self.callback
        elif self.url is not None:
            menu_button['url'] = self.url
        elif self.web_app is not None:
            menu_button['web_app'] = {'url': self.web_app}
        return menu_button


class Menu:

    def __init__(self, button_list: list, buttons_in_line=1, mode='inline', schema_buttons: str = None):
        """

        @param button_list:
        @param buttons_in_line:
        @param mode: inline or keyboard
        @param schema_buttons: example 1:2:1
        """
        self.button_list = button_list
        self.buttons_in_line = buttons_in_line
        self.mode = mode
        self.schema_buttons = schema_buttons

    def build(self):
        menu_items = []
        if isinstance(self.schema_buttons, str) and re.search(r'^([1-9]:)*[1-9]$', self.schema_buttons):
            self.get_buttons_schema(menu_items)
        else:
            i = 0
            line_menu_items = []
            for menu_button in self.button_list:
                i += 1
                line_menu_items.append(menu_button.build())
                if i == self.buttons_in_line:
                    menu_items.append(line_menu_items)
                    i = 0
                    line_menu_items = []
            menu_items.append(line_menu_items)
        if self.mode == 'inline':
            log.info(f'{menu_items}')
            return json.dumps({'inline_keyboard': menu_items})
        else:
            return json.dumps({
                'keyboard': menu_items,
                'resize_keyboard': True,
                'one_time_keyboard': True
            })

    def get_buttons_schema(self, menu_items: list):
        """Get buttons schema"""
        schema = [int(s) for s in self.schema_buttons.split(':')]
        line_menu_items = []
        i = -1
        for menu_button in reversed(self.button_list):
            line_menu_items.insert(0, menu_button.build())
            try:
                schema_key = schema[i]
            except IndexError:
                if len(schema) == 1:
                    i = -1
                else:
                    i = -2
                schema_key = schema[i]
            if len(line_menu_items) == schema_key:
                menu_items.insert(0, line_menu_items)
                i -= 1
                line_menu_items = []
        if line_menu_items:
            menu_items.insert(0, line_menu_items)
        return menu_items



# Generated by Django 4.2.6 on 2024-06-05 06:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0006_user_state_information'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='name',
            field=models.CharField(blank=True, default='', max_length=200, null=True, verbose_name='Имя'),
        ),
        migrations.AlterField(
            model_name='user',
            name='username',
            field=models.CharField(blank=True, default='', max_length=200, null=True, verbose_name='Username'),
        ),
    ]



import json

from django.conf import settings

from oscarbot.bot import Bot
from oscarbot.bot_logger import log


class TGResponse:

    def __init__(self, message: str, menu=None, need_update=True, photo=None, attache=None, video=None,
                 file=None, media_group: list[dict] = None, media_group_type='photo', has_spoiler=False, protect=False,
                 callback_text='', callback_url=False, show_alert=False, cache_time=None,
                 disable_web_page_preview=False, is_delete_message=False) -> None:
        self.tg_bot = None
        self.message = message
        self.menu = menu
        self.attache = attache
        self.need_update = need_update
        self.photo = photo
        self.video = video
        self.file = file
        self.media_group = media_group if media_group else None
        self.media_group_type = media_group_type
        self.has_spoiler = has_spoiler
        self.protect = protect
        self.parse_mode = settings.TELEGRAM_PARSE_MODE if getattr(settings, 'TELEGRAM_PARSE_MODE', None) else 'HTML'
        self.callback_url = callback_url
        self.callback_text = callback_text
        self.show_alert = show_alert
        self.cache_time = cache_time
        self.disable_web_page_preview = disable_web_page_preview
        self.is_delete_message = is_delete_message

    def send(self, token, user=None, content=None, t_id=None):
        self.tg_bot = Bot(token)
        if content and (self.callback_text or self.callback_url):
            self.send_callback(content)
        if self.menu:
            self.menu = self.menu.build()
        data_to_send = {
            'chat_id': user.t_id if user is not None else t_id,
            'message': self.message,
            'reply_keyboard': self.menu,
            'photo': self.photo,
            'video': self.video,
            'protect_content': self.protect,
            'parse_mode': self.parse_mode,
            'disable_web_page_preview': self.disable_web_page_preview,
            'file': self.file,
            'is_delete_message': self.is_delete_message,
        }

        if self.media_group:
            data_to_send['media_group'] = self.media_group
            data_to_send['media_group_type'] = self.media_group_type
            data_to_send['has_spoiler'] = self.has_spoiler
            self.need_update = False

        if content:
            message = content.get('message')
            if not message:
                callback_query = content.get('callback_query')
                message = callback_query.get('message') if callback_query else None
            if message:
                message_id = message.get('message_id')
                data_to_send['message_delete'] = message_id

        if self.need_update and user.last_message_id:
            response_content = self.tg_bot.update_message(**data_to_send, message_id=user.last_message_id)
            response_dict = json.loads(response_content)
            if not response_dict.get('ok') and not self.is_delete_message:
                response_content = self.tg_bot.send_message(**data_to_send)
        else:
            response_content = self.tg_bot.send_message(**data_to_send)
        log.info(f'{response_content}')
        if user:
            user.update_last_sent_message(response_content)

    def can_send(self):
        if self.message is not None or self.video is not None or self.photo is not None:
            return True
        return False

    def send_callback(self, content):
        """Send callback"""
        callback_query = content.get('callback_query') if content else None
        callback_query_id = callback_query.get('id') if callback_query else None
        if callback_query_id:
            params = {
                'callback_query_id': callback_query_id,
                'text': self.callback_text,
                'url': self.callback_url,
                'show_alert': self.show_alert,
                'cache_time': self.cache_time,
            }
            response_content = self.tg_bot.answer_callback_query(**params)
            log.info(response_content)



from django.conf import settings

from oscarbot.menu import Menu
from oscarbot.response import TGResponse
from oscarbot.services import get_bot_model, get_bot_user_model


class QuickBot:

    def __init__(self, user: object, message: str, token: str = None, menu: Menu = None):
        """
        Init QuickBot object for send message to Telegram user
        @param user: user oscarbot.models.User object
        @param message: text message
        @param token: bot token, default get first bot from DB
        @param menu: should be oscarbot.menu.Menu object
        """
        if token:
            self.token = token
        else:
            bot_model = get_bot_model()
            bot_object = bot_model.objects.all().first()
            if bot_object:
                self.token = bot_object.token

        self.user = user
        self.message = message
        self.menu = menu

    def send(self):
        response = TGResponse(message=self.message, menu=self.menu, need_update=False)

        response.send(
            self.token,
            user=self.user
        )



import logging.config

LOGGING_CONFIG = {
    'version': 1,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'default',
        },
    },
    'formatters': {
        'default': {
            'format': '%(name)s - %(levelname)s - %(asctime)s - %(funcName)s: %(lineno)d - %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S',
        },

    },
    'loggers': {
        'oscarbot': {
            'handlers': ['console'],
            'level': logging.DEBUG,
            'propagate': False,
        },
    },

}

logging.config.dictConfig(LOGGING_CONFIG)

log = logging.getLogger('oscarbot')



import importlib

import requests
from django.conf import settings

from oscarbot.action import Action
from oscarbot.bot import Bot
from oscarbot.response import TGResponse
from oscarbot.router import Router
from oscarbot.services import get_bot_user_model
from oscarbot.structures import Message


class BaseHandler:

    # logger: Logger

    def __init__(self, token: str, content: str) -> None:
        self.bot = Bot(token)
        self.content = content
        self.message = Message(content)
        self.user = self.__find_or_create_user_in_db()

    def __find_or_create_user_in_db(self):
        if hasattr(self.message, 'user'):
            first_name = self.message.user.first_name if self.message.user.first_name else ''
            last_name = self.message.user.last_name if self.message.user.last_name else ''
            name = f'{first_name} {last_name}'.strip()
            user_in_db, _ = get_bot_user_model().objects.update_or_create(
                t_id=self.message.user.id,
                defaults={
                    "username": self.message.user.username,
                    "name": name,
                },
            )
            return user_in_db
        return None

    @classmethod
    def __send_do_not_understand(cls):
        need_update, is_delete_message, menu = False, False, None
        message = "Извините, я не понимаю Вас :("
        if getattr(settings, 'NOT_UNDERSTAND_MESSAGE', None):
            message = settings.NOT_UNDERSTAND_MESSAGE
        if getattr(settings, 'NOT_UNDERSTAND_NEED_UPDATE', None):
            need_update = settings.NOT_UNDERSTAND_NEED_UPDATE
        if getattr(settings, 'NOT_UNDERSTAND_IS_DELETE_MESSAGE', None):
            is_delete_message = settings.NOT_UNDERSTAND_IS_DELETE_MESSAGE
        if getattr(settings, 'NOT_UNDERSTAND_MENU', None):
            mod_name, func_name = settings.NOT_UNDERSTAND_MENU.rsplit('.', 1)
            mod = importlib.import_module(mod_name)
            get_menu = getattr(mod, func_name)
            menu = get_menu()
        return TGResponse(message=message, menu=menu, need_update=need_update, is_delete_message=is_delete_message)

    def __work_text_processor(self, photo=None):
        if getattr(settings, 'TELEGRAM_TEXT_PROCESSOR', None):
            response = self.__get_text_handler(photo=photo)
            if response:
                return response
        return self.__send_do_not_understand()

    def handle(self) -> TGResponse:
        if hasattr(self.message, 'data') and self.message.data:
            return self.__handle_callback_data(self.message.data)
        elif hasattr(self.message, 'text') and self.message.text:
            return self.__handle_text_data()
        elif hasattr(self.message, 'photo') and self.message.photo:
            return self.__handle_photo_data()
        elif hasattr(self.message, 'document') and self.message.document:
            return self.__handle_document_data()
        elif hasattr(self.message, 'voice') and self.message.voice:
            return self.__handle_voice_data()
        return self.__work_text_processor()

    def __handle_voice_data(self):
        if getattr(settings, 'TELEGRAM_VOICE_PROCESSOR', None):
            mod_name, func_name = settings.TELEGRAM_VOICE_PROCESSOR.rsplit('.', 1)
            mod = importlib.import_module(mod_name)
            audio_processor = getattr(mod, func_name)
            voice_file = self.bot.get_file(self.message.voice.get('file_id'))
            data = {
                'voice': voice_file,
            }
            response = audio_processor(self.user, data)
            if response:
                return response
        return self.__send_do_not_understand()

    def __get_text_handler(self, photo=None):
        mod_name, func_name = settings.TELEGRAM_TEXT_PROCESSOR.rsplit('.', 1)
        mod = importlib.import_module(mod_name)
        text_processor = getattr(mod, func_name)
        data = {
            'text': self.message.text,
            'photo': photo,
        }
        response = text_processor(self.user, data)
        if response:
            return response

        return False

    def __handle_callback_data(self, path):
        router = Router(path)
        func, arguments = router()
        if func:
            response = func(self.user, **arguments)
            if response:
                self.user.update_path(path)
                return response
        return self.__send_do_not_understand()

    def __handle_text_data(self):
        if self.message.text[0] == '/':
            return self.__handle_callback_data(self.message.text)

        if self.user.want_action:
            action = Action(self.user, self.message.text)
            return action()

        return self.__work_text_processor()

    def __handle_photo_data(self):
        """ WIP: """
        photos = []
        for file in self.message.photo:
            file_id = file['file_id']
            res = requests.get(f'{settings.TELEGRAM_URL}{self.bot.token}/getFile?file_id={file_id}')
            file_path = res.json()['result']['file_path']
            photos.append(
                f'https://api.telegram.org/file/bot{self.bot.token}/{file_path}'
            )
        return self.__work_text_processor(photo=photos)

    def __handle_document_data(self):
        """ WIP: """
        return self.__work_text_processor()



import importlib
from inspect import getmembers

from django.conf import settings


class Action:

    def __init__(self, user, message):
        self.actions = self.__collect_all_actions()
        self.user = user
        self.message = message

    def __call__(self):
        if self.user.want_action:
            if self.user.want_action in self.actions.keys():
                action = self.actions[self.user.want_action]
                if action:
                    return action(self.user, self.message)
        self.user.want_action = None
        self.user.save()
        return None

    @staticmethod
    def __collect_all_actions() -> dict:
        all_actions = {}
        if len(settings.OSCARBOT_APPS) > 0:
            for app in settings.OSCARBOT_APPS:
                try:
                    module_name = f'{app}.actions'
                    action_item = importlib.import_module(module_name)
                    for action in dir(action_item):
                        if action.startswith('action__'):
                            all_actions[f'{app}.{action}'] = getattr(action_item, action)
                            all_actions[f'{app}.{action[8:]}'] = getattr(action_item, action)
                except ImportError as ex:
                    print(ex)
        return all_actions
